@@ -1,4 +1,3 @@
-import shutil
 import subprocess
 import unittest
 import tempfile
@@ -11,15 +10,16 @@ import jinja2
 
 from bigflow.cli import cli
 from bigflow.scaffold import templating as tt
+from bigflow.scaffold import infra
+
+from test import mixins
 
 
-class ProjectScaffoldE2ETestCase(TestCase):
-    def tearDown(self):
-        clear_project_leftovers(Path('my_project_project'))
-
-
-class ProjectScaffoldE2E(ProjectScaffoldE2ETestCase):
-
+class ProjectScaffoldE2E(
+    mixins.FileUtilsMixin,
+    mixins.TempCwdMixin,
+    unittest.TestCase,
+):
     maxDiff = 5000
 
     @mock.patch('bigflow.cli.gcloud_project_list')
@@ -48,7 +48,7 @@ class ProjectScaffoldE2E(ProjectScaffoldE2ETestCase):
         cli(['start-project'])
 
         # then
-        self.scaffolded_basic_project_should_have_one_environment()
+        self.check_scaffolded_basic_project()
 
         # and
         self.scaffolded_project_tests_should_work()
@@ -85,56 +85,12 @@ class ProjectScaffoldE2E(ProjectScaffoldE2ETestCase):
         res = subprocess.run("python -m unittest discover -s test -p '*.py'", cwd=cwd, check=True, capture_output=True, shell=True)
         self.assertRegexpMatches(res.stderr.decode(), r"OK$")
 
-    def scaffolded_basic_project_should_have_one_environment(self):
-        self.check_file_content(Path('my_project_project') / 'deployment_config.py', '''from bigflow.configuration import DeploymentConfig
+    def check_scaffolded_basic_project(self):
 
-deployment_config = DeploymentConfig(
-    name='dev',
-    properties={
-       'docker_repository': 'test_repository',
-       'gcp_project_id': 'my_gcp_project',
-       'dags_bucket': 'my_gcp_bucket'})
-''')
-        self.check_file_content(Path('my_project_project') / 'my_project' / 'wordcount' / 'pipeline.py', '''import uuid
-import logging
-
-from bigflow.configuration import Config
-from bigflow.build.reflect import materialize_setuppy
-from apache_beam.options.pipeline_options import SetupOptions, StandardOptions, WorkerOptions, GoogleCloudOptions, \
-    PipelineOptions
-
-logger = logging.getLogger(__name__)
-
-workflow_config = Config(
-    name='dev',
-    properties={
-        'gcp_project_id': 'my_gcp_project',
-        'staging_location': 'my_project/beam_runner/staging',
-        'temp_location': 'my_project/beam_runner/temp',
-        'region': 'europe-west1',
-        'machine_type': 'n1-standard-1'}).resolve()
-
-
-def dataflow_pipeline_options():
-    options = PipelineOptions()
-
-    google_cloud_options = options.view_as(GoogleCloudOptions)
-    google_cloud_options.project = workflow_config['gcp_project_id']
-    google_cloud_options.job_name = f'beam-wordcount-{uuid.uuid4()}'
-    google_cloud_options.staging_location = f"gs://{workflow_config['staging_location']}"
-    google_cloud_options.temp_location = f"gs://{workflow_config['temp_location']}"
-    google_cloud_options.region = workflow_config['region']
-    # google_cloud_options.service_account_email = 'your-service-account'
-
-    options.view_as(WorkerOptions).machine_type = workflow_config['machine_type']
-    options.view_as(WorkerOptions).max_num_workers = 2
-    options.view_as(WorkerOptions).autoscaling_algorithm = 'THROUGHPUT_BASED'
-    options.view_as(StandardOptions).runner = 'DataflowRunner'
-
-    options.view_as(SetupOptions).setup_file = str(materialize_setuppy().absolute())
-
-    logger.info(f"Run beam pipeline with options {str(options)}")
-    return options''')
+        self.assertFileExists("my_project_project/my_project/wordcount/pipeline.py")
+        self.assertFileContentRegex(
+            "my_project_project/deployment_config.py",
+            r"""deployment_config = DeploymentConfig\(""")
 
     def scaffolded_advanced_project_should_have_three_environments(self):
         self.check_file_content(Path('my_project_project') / 'deployment_config.py', '''from bigflow.configuration import Config
@@ -213,10 +169,6 @@ internationalports_workflow = Workflow(
             self.assertEqual(f.read(), template)
 
 
-def clear_project_leftovers(image_dir: Path):
-    shutil.rmtree(image_dir, ignore_errors=True)
-
-
 class TemplatingTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -273,7 +225,6 @@ class TemplatingTestCase(unittest.TestCase):
         ]:
             self.assertEqual(self.readfile(expected_file), "OK")
 
-
     def test_conditional_render(self):
         # given
         loader = jinja2.DictLoader({
@@ -302,4 +253,130 @@ class TemplatingTestCase(unittest.TestCase):
         for should_not_exist in [
             "a1", "b2", "c2",
         ]:
-            self.assertFalse((Path(self.tmp_dir) / should_not_exist).exists())
\ No newline at end of file
+            self.assertFalse((Path(self.tmp_dir) / should_not_exist).exists())
+
+
+class CloudComposerCreatorTestCase(unittest.TestCase):
+
+    infrastructure_parameters = (
+        'fake-gcp-project',
+        'fake-bigflow-project',
+        'dev',
+        'europe-west2',
+        'europe-west2-b'
+    )
+
+    @mock.patch('bigflow.scaffold.infra.run_process')
+    def test_should_create_cloud_composer(self, run_process_mock):
+        # given
+        gcp_project_id, bigflow_project_name, environment_name, region, zone = self.infrastructure_parameters
+
+        # when
+        infra.try_create(infra.CloudComposer(
+            gcp_project_id,
+            bigflow_project_name,
+            environment_name,
+            region,
+            zone
+        ))
+
+        # then
+        expected_router_name = f'bigflow-router-{bigflow_project_name}-{environment_name}-{region}'
+        run_process_mock.assert_has_calls([
+            mock.call([
+                'gcloud', 'compute',
+                '--project', gcp_project_id,
+                'routers', 'create', expected_router_name,
+                '--network', 'default',
+                '--region', region
+            ]),
+            mock.call([
+                'gcloud', 'compute',
+                '--project', gcp_project_id,
+                'routers', 'nats', 'create', f'bigflow-nat-{bigflow_project_name}-{environment_name}-{region}',
+                f'--router={expected_router_name}',
+                '--auto-allocate-nat-external-ips',
+                '--nat-all-subnet-ip-ranges',
+                '--enable-logging'
+            ]),
+            mock.call([
+                'gcloud', 'composer',
+                '--project', gcp_project_id,
+                'environments', 'create', f'bigflow-composer-{bigflow_project_name}-{environment_name}-{region}',
+                f'--location={region}',
+                f'--zone={zone}',
+                f'--env-variables=env={environment_name}',
+                f'--machine-type=n1-standard-2',
+                f'--node-count=3',
+                f'--python-version=3',
+                f'--enable-ip-alias',
+                f'--network=default',
+                f'--subnetwork=default',
+                f'--enable-private-environment',
+                f'--enable-private-endpoint',
+            ])
+        ])
+
+    @mock.patch('bigflow.scaffold.infra.run_process')
+    @mock.patch('bigflow.scaffold.infra._composer_create_command')
+    def test_should_destroy_leftovers_when_error_occurs_during_creation(self, composer_create_command_mock, run_process_mock):
+        # given
+        def raise_error(*args):
+            raise ValueError("Fail creation of composer command")
+
+        gcp_project_id, bigflow_project_name, environment_name, region, zone = self.infrastructure_parameters
+        composer_create_command_mock.side_effect = raise_error
+
+        # when
+        infra.try_create(infra.CloudComposer(
+            gcp_project_id,
+            bigflow_project_name,
+            environment_name,
+            region,
+            zone
+        ))
+
+        # then
+        expected_router_name = f'bigflow-router-{bigflow_project_name}-{environment_name}-{region}'
+        expected_composer_name = f'bigflow-composer-{bigflow_project_name}-{environment_name}-{region}'
+        expected_nat_name = f'bigflow-nat-{bigflow_project_name}-{environment_name}-{region}'
+
+        run_process_mock.assert_has_calls([
+            mock.call([
+                'gcloud', 'compute',
+                '--project', gcp_project_id,
+                'routers', 'create', expected_router_name,
+                '--network', 'default',
+                '--region', region
+            ]),
+            mock.call([
+                'gcloud', 'compute',
+                '--project', gcp_project_id,
+                'routers', 'nats', 'create', f'bigflow-nat-{bigflow_project_name}-{environment_name}-{region}',
+                f'--router={expected_router_name}',
+                '--auto-allocate-nat-external-ips',
+                '--nat-all-subnet-ip-ranges',
+                '--enable-logging'
+            ]),
+            mock.call([
+                'gcloud', 'composer',
+                'environments', 'delete', expected_composer_name,
+                '--location', region,
+                '--project', gcp_project_id,
+                '--quiet'
+            ]),
+            mock.call([
+                'gcloud', 'compute',
+                '--project', gcp_project_id,
+                'routers', 'nats', 'delete', expected_nat_name,
+                '--router', expected_router_name,
+                '--quiet'
+            ]),
+            mock.call([
+                'gcloud', 'compute',
+                '--project', gcp_project_id,
+                'routers', 'delete', expected_router_name,
+                '--region', region,
+                '--quiet'
+            ])
+        ])
\ No newline at end of file
